
from ..core import classes
from ..utils import EQ, NE, LT, LE, GT, GE, SYMBOL, AND, NOT, OR, NUMBER, IN, NOTIN
from ..basealgebra import Algebra, Verbatim

head_mth_map = {
    EQ: Algebra.__eq__,
    NE: Algebra.__ne__,
    LT: Algebra.__lt__,
    LE: Algebra.__le__,
    GT: Algebra.__gt__,
    GE: Algebra.__ge__,
    }

class Logic(Algebra):
    """ Represents n-ary predicate expressions.

    An expression ``predicate(*args)`` is hold in a pair::

      (<predicate constant or function>, args)

    Examples of predicates::

      Logic(NUMBER, True)  - 0-ary predicate, represents truth value True
      Logic(NUMBER, False) - 0-ary predicate, represents truth value False
      Logic(SYMBOL, 'x')   - 0-ary predicate, represents a boolean symbol x
      Logic(NOT, x)        - 1-ary predicate, represents ``not x`` boolean expression
      Logic(AND, frozenset([x,y,z])) - represents ``x and y and z`` boolean expression
      Logic(OR, frozenset([x,y])) - represents boolean expression ``x or y``
      Logic(LT, (x, y))    - represents relation ``x < y``
      Logic(LE, (x, y))    - represents relation ``x <= y``
      Logic(GT, (x, y))    - represents relation ``x > y``
      Logic(GT, (x, y))    - represents relation ``x >= y``
      Logic(EQ, (x, y))    - represents relation ``x == y``
      Logic(NE, (x, y))    - represents relation ``x != y``
      Logic(IN, (x, y))    - represents relation ``x in y``
      Logic(NOTIN, (x, y)) - represents relation ``x not in y``

    """

    @classmethod
    def get_operand_algebra(cls, head, index=0):
        """ Return algebra class for index-th operand of operation with head.
        """
        if head in [AND, OR, NOT]:
            return cls
        if head in [LT, LE, GT, GE, EQ, NE]:
            return classes.Calculus
        if head in [IN, NOTIN]:
            if index==1:
                return classes.Set
            return classes.Verbatim
        raise NotImplementedError('Algebra %s does not support operation %s' % (cls.__name__, head))

    def __nonzero__(self, head_mth_get=head_mth_map.get):
        """ Return lexicographic truth value for relational predicates
        and True for non-numeric predicates.
        """
        head, data = self.pair
        return head.nonzero(type(self), data)
        mth = head_mth_get(head)
        if mth is None:
            if head is NUMBER:
                return data
            return True
        return mth(*data)

    def as_verbatim(self):
        return Verbatim(self.head, self.data)

    @classmethod
    def get_predefined_symbols(cls, name):
        if name=='True': return cls.true
        if name=='False': return cls.false
        return

    @classmethod
    def convert_number(cls, obj, typeerror=True):
        if type(obj) is bool:
            return obj
        return cls.handle_convert_failure('number', obj, typeerror, 'expected bool')

    def convert_operand(self, obj, typeerror=True):
        head = self.head
        if head in [LT, GT, LE, GE, EQ, NE]:
            return classes.Calculus.convert(obj, typeerror)
        if isinstance(obj, bool):
            return type(self)(NUMBER, obj)
        return self.convert(obj, typeerror)

    is_Lt = property(lambda self: self.head is LT)
    is_Le = property(lambda self: self.head is LE)
    is_Gt = property(lambda self: self.head is GT)
    is_Ge = property(lambda self: self.head is GE)
    is_Eq = property(lambda self: self.head is EQ)
    is_Ne = property(lambda self: self.head is NE)
    is_And = property(lambda self: self.head is AND)
    is_Or = property(lambda self: self.head is OR)
    is_Not = property(lambda self: self.head is NOT)

    def _subs(self, subexpr, newexpr):
        head, data = self.pair
        cls = type(self)

        if type(subexpr) is not cls:
            if head is SYMBOL or head is NUMBER:
                if subexpr==data:
                    return self.convert_operand(newexpr)
                return self
        else:
            h, d = subexpr.pair
            if h is head and d==data:
                return self.convert_operand(newexpr)

        if head is SYMBOL or head is NUMBER:            
            return self
        if head is AND:
            return cls.And(*[a._subs(subexpr, newexpr) for a in data])        
        if head is OR:
            return cls.Or(*[a._subs(subexpr, newexpr) for a in data])
        if head is NOT:
            return cls.Not(data._subs(subexpr, newexpr))
        if head is LT:
            return cls.Lt(*[a._subs(subexpr, newexpr) for a in data])
        if head is GT:
            return cls.Gt(*[a._subs(subexpr, newexpr) for a in data])
        if head is LE:
            return cls.Le(*[a._subs(subexpr, newexpr) for a in data])
        if head is GE:
            return cls.Ge(*[a._subs(subexpr, newexpr) for a in data])
        if head is EQ:
            return cls.Eq(*[a._subs(subexpr, newexpr) for a in data])
        if head is NE:
            return cls.Ne(*[a._subs(subexpr, newexpr) for a in data])
        return self

    @classmethod
    def Lt(cls, *seq):
        if len(seq)==2:
            lhs, rhs = seq
            if lhs.head is NUMBER and rhs.head is NUMBER:
                return cls.true if lhs.data < rhs.data else cls.false
            if not lhs < rhs:
                return cls(GT, (rhs, lhs))
        return cls(LT, seq)

    @classmethod
    def Le(cls, *seq):
        if len(seq)==2:
            lhs, rhs = seq
            if lhs.head is NUMBER and rhs.head is NUMBER:
                return cls.true if lhs.data <= rhs.data else cls.false
            if not lhs < rhs:
                return cls(GE, (rhs, lhs))
        return cls(LE, seq)

    @classmethod
    def Gt(cls, *seq):
        if len(seq)==2:
            lhs, rhs = seq
            if lhs.head is NUMBER and rhs.head is NUMBER:
                return cls.true if lhs.data > rhs.data else cls.false
            if not lhs < rhs:
                return cls(LT, (rhs, lhs))
        return cls(GT, seq)

    @classmethod
    def Ge(cls, *seq):
        if len(seq)==2:
            lhs, rhs = seq
            if lhs.head is NUMBER and rhs.head is NUMBER:
                return cls.true if lhs.data >= rhs.data else cls.false
            if not lhs < rhs:
                return cls(LE, (rhs, lhs))
        return cls(GE, seq)

    @classmethod
    def Eq(cls, *seq):
        if len(seq)==2:
            lhs, rhs = seq
            if lhs==rhs:
                return cls.true
            if lhs.head is NUMBER and rhs.head is NUMBER:
                return cls.false
        return cls(EQ, seq)

    @classmethod
    def Ne(cls, *seq):
        if len(seq)==2:
            lhs, rhs = seq
            if lhs==rhs:
                return cls.false
            if lhs.head is NUMBER and rhs.head is NUMBER:
                return cls.true
        return cls(NE, seq)

    @classmethod
    def Element(cls, element, container):
        if hasattr(container, 'contains'):
            return container.contains(element)
        return cls(IN, (element, container))

    @classmethod
    def Contains(cls, container, element):
        return cls.IsElement(element, container)

    #XXX: IsSubset

    @classmethod
    def Or(cls, *seq):
        s = set([])
        for a in seq:
            assert type(a) is cls,`type(a)`
            head, data = a.pair
            if head is NUMBER:
                if data:
                    return cls.true
            elif head is OR:
                s.update(data)
            elif head is NOT:
                if data in s:
                    return cls.true
                s.add(a)
            else:
                if cls(NOT, a) in s:
                    return cls.true
                s.add(a)
        if not s:
            return cls.false
        if len(s)==1:
            return s.pop()
        return cls(OR, frozenset(s))

    @classmethod
    def And(cls, *seq):
        s = set([])
        for a in seq:
            assert type(a) is cls,`type(a)`
            head, data = a.pair
            if head is NUMBER:
                if not data:
                    return cls.false
            elif head is AND:
                s.update(data)
            elif head is NOT:
                if data in s:
                    return cls.false
                else:
                    s.add(a)
            else:
                if cls(NOT,a) in s:
                    return cls.false
                s.add(a)
        if not s:
            return cls.true
        if len(s)==1:
            return s.pop()
        return cls(AND, frozenset(s))

    @classmethod
    def Not(cls, obj):
        assert type(obj) is cls,`type(obj)`
        head, data = obj.pair
        if head is NUMBER: return cls(NUMBER, not data)
        if head is NOT: return data
        if head is NE: return cls(EQ, data)
        if head is EQ: return cls(NE, data)
        if head is LT: return cls(GE, data)
        if head is GT: return cls(LE, data)
        if head is LE: return cls(GT, data)
        if head is GE: return cls(LT, data)
        return cls(NOT, obj)

    #XXX: Xor, Implies, Equiv

Logic.one = Logic.true = Logic(NUMBER, True)
Logic.zero = Logic.false = Logic(NUMBER, False)
classes.Logic = Logic



from pymaclab.filters._bkfilter import bkfilter as bkf
import numpy as np

def bkfilter(data=None,up=6,dn=32,kkl=12):
    if type(data) != type(np.array([1,2,3])):
        data = np.array(data)
    tlen = len(data)
    return bkf(data,up,dn,kkl,tlen)



# Import the pymaclab module into its namespace, also import os module
import pymaclab as pm
from pymaclab.modfiles import models
from copy import deepcopy

def test8():

    # Test the steady state method in which USE_FOCS was declared inside the model
    rbc_focs = pm.newMOD(models.stable.rbc1_focs,mesg=True)
    # Did it work?
    assert 'sstate' in dir(rbc_focs)
    
    # Now copy the sstate dictionary and test the steady state method in which it is passed from outside the model
    statedic = deepcopy(rbc_focs.sstate)
    rbc_ext = pm.newMOD(models.stable.rbc1_extss,mesg=True,sstate=statedic)
    # Did it work?
    assert 'sstate' in dir(rbc_ext)
    




__all__ = ['APPLY']

from .base import Head, heads_precedence
from .functional import FunctionalHead

from ..core import init_module, Expr
init_module.import_heads()
init_module.import_numbers()
init_module.import_lowlevel_operations()

class ApplyHead(FunctionalHead):
    """
    ApplyHead is a head for n-ary apply operation,
    data is a (1+n)-tuple of expression operands
    """
    op_mth = '__call__'
    parenthesis = '()'

    def is_data_ok(self, cls, data):
        if isinstance(data, tuple) and len(data)==2:
            func, args = data
            fcls = cls.get_function_algebra()
            if not isinstance(func, fcls):
                return '%s data[0] must be %s instance but got %s' % (self, fcls, type(func))
            msg = func.head.is_data_ok(type(func), func.data)
            if msg:
                return '%s data=%r: %s' % (func.head, func.pair, msg)
            if type(args) is tuple:
                for i,a in enumerate(args):
                    acls = func.get_argument_algebra(i)
                    if not isinstance(a, Expr) or type(a) is not acls:
                        return '%s data[1][%s] must be %s instance but got %s' % (self, i, acls, type(a))
            else:
                return '%s data[1] must be tuple but got %s' % (self, type(args))
        else:
            return '%s data instance must be 2-tuple' % (self)

    def __repr__(self): return 'APPLY'

    def new(self, cls, (func, args), evaluate=True):
        if not isinstance(func, Expr):
            fcls = cls.get_function_algebra()
            func = fcls(CALLABLE, func)
        return cls(APPLY, (func, args))

    def commutative_mul_number(self, cls, lhs, rhs):
        return term_coeff_new(cls, (lhs, rhs))

    def pow(self, cls, base, exp):
        return POW.new(cls, (base, exp))

    pow_number = pow

    def scan(self, proc, cls, data, target):
        f, args = data
        f.head.scan(proc, cls, f.data, target)
        for arg in args:
            arg.head.scan(proc, cls, arg.data, target)
        proc(cls, self, data, target)

    def walk(self, func, cls, data, target):
        f, args = data
        new_f = f.head.walk(func, cls, f.data, f)
        new_args = []
        flag = f is not new_f
        for arg in args:
            h, d = arg.pair
            new_arg = arg.head.walk(func, cls, arg.data, arg)
            if arg is not new_arg:
                flag = True
            new_args.append(new_arg)
        if flag:
            new_args = tuple(new_args)
            if new_f.head is CALLABLE:
                r = new_f.data(*new_args)
            else:
                r = cls(APPLY, (new_f, new_args))
            return func(cls, r.head, r.data, r)
        return func(cls, self, data, target)

    def expand(self, cls, expr):
        f, args = expr.data
        new_f = f
        new_args = [a.expand() for a in args]
        return new_f (*new_args)
        return expr

    def expand_intpow(self, cls, base, exp):
        return cls(POW, (base, exp))

    def diff(self, cls, data, expr, symbol, order, cache={}):
        key = (expr, symbol, order)
        result = cache.get(key)
        if result is not None:
            return result
        if isinstance(order, inttypes):
            pass            
        key1 = (expr, symbol, 1)
        result = cache.get(key1)
        if result is None:
            func, args = data
            fcls = cls.get_function_algebra()
            dcls = fcls.get_differential_algebra()
            if len(args)==1:
                arg = args[0]
                da = arg.head.diff(cls, arg.data, arg, symbol, 1, cache=cache)
                if symbol not in da.symbols_data:
                    # argument is linear with respect to symbol
                    df = func.head.fdiff(fcls, func.data, func, 0, order)
                    if df is NotImplemented:
                        if isinstance(order, int):
                            df = func.head.fdiff(fcls, func.data, func, 0, 1)
                            result = df(*args) * da
                            if order>1:
                                result = result.head.diff(cls, result.data, result, symbol, order-1, cache=cache)
                        else:
                            d = dcls(FDIFF, cls(NUMBER, 0))**order
                            df = fcls(APPLY, (d, (func,)))
                            result = df(*args) * da**order
                    else:
                        result = df(*args) * da**order
                elif isinstance(order, int):
                    df = func.head.fdiff(fcls, func.data, func, 0, 1)
                    result = df(*args) * da
                    if order>1:
                        result = result.head.diff(cls, result.data, result, symbol, order-1, cache=cache)
                else:
                    d = dcls(DIFF, cls(NUMBER, 0))**order
                    result = cls(APPLY, (d, (expr,)))
                cache[key] = result
                return result
            else:
                result = cls(NUMBER, 0)
                for i in range(len(args)):
                    arg = args[i]
                    da = arg.head.diff(cls, arg.data, arg, symbol, 1, cache=cache)
                    df = func.head.fdiff(fcls, func.data, func, i, 1)
                    result += df(*args) * da
            cache[key1] = result
        if order>1:
            result = result.head.diff(cls, result.data, result, symbol, order-1, cache=cache)
            cache[key] = result
        return result

    def apply(self, cls, data, func, args):
        return cls(APPLY, (func, args))

    def integrate_indefinite(self, cls, data, expr, x):
        if x not in expr.symbols_data:
            return expr * cls(SYMBOL, x)
        return cls(INTEGRAL_INDEFINITE, (expr, x))

    def integrate_definite(self, cls, data, expr, x, a, b):
        if x not in expr.symbols_data:
            return expr * (b - a)
        return cls(INTEGRAL_DEFINITE, (expr, x, a, b))

    

APPLY = ApplyHead()



from __future__ import with_statement

from sympycore import Verbatim, SymbolicEquality, heads

def test_basic():
    a = Verbatim('a')
    assert Verbatim.convert('a')==a
    assert Verbatim.convert(a) == a

    assert repr(a) == "Verbatim(SYMBOL, 'a')"
    
    assert Verbatim.convert('1') == Verbatim(heads.NUMBER, 1)
    assert Verbatim.convert(1) == Verbatim(heads.NUMBER, 1)
    assert Verbatim.convert(None) == Verbatim(heads.SPECIAL, None)
    assert Verbatim.convert(Ellipsis) == Verbatim(heads.SPECIAL, Ellipsis)
    assert Verbatim.convert(['a']) == Verbatim(heads.LIST, (Verbatim(heads.SYMBOL, 'a'), )),`Verbatim.convert(['a'])`
    assert Verbatim.convert(('a','1')) == Verbatim(heads.TUPLE, (Verbatim(heads.SYMBOL, 'a'), Verbatim(heads.NUMBER, 1))),`Verbatim.convert(('a',1))`
    assert Verbatim.convert({1:'2'}) == Verbatim(heads.DICT, ((Verbatim(heads.NUMBER, 1), Verbatim(heads.NUMBER, 2)),)),`Verbatim.convert({1:2})`
    assert str(Verbatim(heads.SPECIAL, None))=='None'
    assert str(Verbatim(heads.SPECIAL, Ellipsis)) in ['...', 'Ellipsis'],`str(Verbatim(heads.SPECIAL, Ellipsis))`

def test_operations():
    a = Verbatim('a')
    b = Verbatim('b')
    c = Verbatim('c')
    d = Verbatim('d')
    assert +a == Verbatim('+a'),`+a,Verbatim('+a')`

    assert str(+a) in ['+a','a'],`str(+a)`
    assert repr(+a) == "Verbatim(POS, Verbatim(SYMBOL, 'a'))",repr(+a)
    assert repr(-a) == "Verbatim(NEG, Verbatim(SYMBOL, 'a'))",repr(-a)
    assert repr(~a) == "Verbatim(INVERT, Verbatim(SYMBOL, 'a'))",repr(~a)

    assert repr(a+b)== "Verbatim(ADD, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a+b)
    assert repr(a-b)== "Verbatim(SUB, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a-b)
    assert repr(a*b)== "Verbatim(MUL, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a*b)
    assert repr(a/b)== "Verbatim(DIV, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a/b)
    assert repr(a//b)== "Verbatim(FLOORDIV, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a//b)
    assert repr(a**b)== "Verbatim(POW, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a**b)
    assert repr(a%b)== "Verbatim(MOD, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a%b)
    assert repr(a|b)== "Verbatim(BOR, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a|b)
    assert repr(a&b)== "Verbatim(BAND, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a&b)
    assert repr(a^b)== "Verbatim(BXOR, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a^b)
    assert repr(a<<b)== "Verbatim(LSHIFT, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a<<b)
    assert repr(a>>b)== "Verbatim(RSHIFT, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(a>>b)
    assert repr(divmod(a,b))== "Verbatim(APPLY, (Verbatim(CALLABLE, <built-in function divmod>), (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b'))))", repr(divmod(a,b))
    assert repr(a(b))== "Verbatim(APPLY, (Verbatim(SYMBOL, 'a'), (Verbatim(SYMBOL, 'b'),)))", repr(a(b))
    assert repr(a())== "Verbatim(APPLY, (Verbatim(SYMBOL, 'a'), ()))", repr(a())
    assert repr(a[b])== "Verbatim(SUBSCRIPT, (Verbatim(SYMBOL, 'a'), (Verbatim(SYMBOL, 'b'),)))", repr(a[b])
    assert repr(a[b,c])== "Verbatim(SUBSCRIPT, (Verbatim(SYMBOL, 'a'), (Verbatim(SYMBOL, 'b'), Verbatim(SYMBOL, 'c'))))", repr(a[b,c])
    assert repr(a[b:c])== "Verbatim(SUBSCRIPT, (Verbatim(SYMBOL, 'a'), (Verbatim(SLICE, (Verbatim(SYMBOL, 'b'), Verbatim(SYMBOL, 'c'), Verbatim(SPECIAL, None))),)))", repr(a[b:c])
    assert repr(a[:])== "Verbatim(SUBSCRIPT, (Verbatim(SYMBOL, 'a'), (Verbatim(SLICE, (Verbatim(SPECIAL, None), Verbatim(SPECIAL, None), Verbatim(SPECIAL, None))),)))", repr(a[:])
    assert repr(a[b:])== "Verbatim(SUBSCRIPT, (Verbatim(SYMBOL, 'a'), (Verbatim(SLICE, (Verbatim(SYMBOL, 'b'), Verbatim(SPECIAL, None), Verbatim(SPECIAL, None))),)))", repr(a[b:])
    assert repr(a[:b])== "Verbatim(SUBSCRIPT, (Verbatim(SYMBOL, 'a'), (Verbatim(SLICE, (Verbatim(SPECIAL, None), Verbatim(SYMBOL, 'b'), Verbatim(SPECIAL, None))),)))", repr(a[:b])
    assert repr(a[::b])== "Verbatim(SUBSCRIPT, (Verbatim(SYMBOL, 'a'), (Verbatim(SLICE, (Verbatim(SPECIAL, None), Verbatim(SPECIAL, None), Verbatim(SYMBOL, 'b'))),)))", repr(a[::b])

    with SymbolicEquality(Verbatim):
        assert repr(a==a)=="Logic('a==a')",repr(a==a)
    
    assert repr(Verbatim('a and b'))=="Verbatim(AND, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a and b'))
    assert repr(Verbatim('a or b'))=="Verbatim(OR, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a or b'))
    assert repr(Verbatim('a in b'))=="Verbatim(IN, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a in b'))
    assert repr(Verbatim('a not in b'))=="Verbatim(NOTIN, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a not in b'))
    assert repr(Verbatim('a is b'))=="Verbatim(IS, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a is b'))
    assert repr(Verbatim('a is not b'))=="Verbatim(ISNOT, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a is not b'))
    assert repr(Verbatim('a and b and c'))=="Verbatim(AND, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b'), Verbatim(SYMBOL, 'c')))", repr(Verbatim('a and b and c'))
    assert repr(Verbatim('a or b or c'))=="Verbatim(OR, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b'), Verbatim(SYMBOL, 'c')))", repr(Verbatim('a or b or c'))
    assert repr(Verbatim('(a)'))=="Verbatim(SYMBOL, 'a')", repr(Verbatim('(a)'))
    assert repr(Verbatim('()'))=="Verbatim(TUPLE, ())", repr(Verbatim('()'))
    assert repr(Verbatim('(a,)'))=="Verbatim(TUPLE, (Verbatim(SYMBOL, 'a'),))", repr(Verbatim('(a,)'))
    assert repr(Verbatim('(a,b)'))=="Verbatim(TUPLE, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('(a,b)'))
    assert repr(Verbatim('[]'))=="Verbatim(LIST, ())", repr(Verbatim('[]'))
    assert repr(Verbatim('[a]'))=="Verbatim(LIST, (Verbatim(SYMBOL, 'a'),))", repr(Verbatim('[a]'))
    assert repr(Verbatim('[a,b]'))=="Verbatim(LIST, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('[a,b]'))
    assert repr(Verbatim('lambda : a'))=="Verbatim(LAMBDA, ((), Verbatim(SYMBOL, 'a')))", repr(Verbatim('lambda : a'))
    assert repr(Verbatim('lambda x: a'))=="Verbatim(LAMBDA, ((Verbatim(SYMBOL, 'x'),), Verbatim(SYMBOL, 'a')))", repr(Verbatim('lambda x: a'))
    assert repr(Verbatim('a==b'))=="Verbatim(EQ, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a==b'))
    assert repr(Verbatim('a!=b'))=="Verbatim(NE, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a!=b'))
    assert repr(Verbatim('a<b'))=="Verbatim(LT, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a<b'))
    assert repr(Verbatim('a>b'))=="Verbatim(GT, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a>b'))
    assert repr(Verbatim('a<=b'))=="Verbatim(LE, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a<=b'))
    assert repr(Verbatim('a>=b'))=="Verbatim(GE, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a>=b'))
    assert repr(Verbatim('a<b<c'))=="Verbatim(AND, (Verbatim(LT, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b'))), Verbatim(LT, (Verbatim(SYMBOL, 'b'), Verbatim(SYMBOL, 'c')))))", repr(Verbatim('a<b<c'))
    assert repr(Verbatim('a[b]'))=="Verbatim(SUBSCRIPT, (Verbatim(SYMBOL, 'a'), (Verbatim(SYMBOL, 'b'),)))", repr(Verbatim('a[b]'))
    assert repr(Verbatim('a[b,c]'))=="Verbatim(SUBSCRIPT, (Verbatim(SYMBOL, 'a'), (Verbatim(SYMBOL, 'b'), Verbatim(SYMBOL, 'c'))))", repr(Verbatim('a[b,c]'))

    assert repr(Verbatim('divmod(a,b)'))=="Verbatim(APPLY, (Verbatim(SYMBOL, 'divmod'), (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b'))))",repr(Verbatim('divmod(a,b)'))
    assert repr(Verbatim('slice(a)'))=="Verbatim(SLICE, (Verbatim(SYMBOL, 'a'),))",repr(Verbatim('slice(a)'))
    assert repr(Verbatim('slice(a,b)'))=="Verbatim(SLICE, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))",repr(Verbatim('slice(a,b)'))
    assert repr(Verbatim('slice(a,b,c)'))=="Verbatim(SLICE, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b'), Verbatim(SYMBOL, 'c')))",repr(Verbatim('slice(a,b,c)'))
    assert repr(Verbatim('a[b:c,d]'))=="Verbatim(SUBSCRIPT, (Verbatim(SYMBOL, 'a'), (Verbatim(SLICE, (Verbatim(SYMBOL, 'b'), Verbatim(SYMBOL, 'c'), Verbatim(SPECIAL, None))), Verbatim(SYMBOL, 'd'))))", repr(Verbatim('a[b:c,d]'))
    assert repr(Verbatim('a[b:c]'))=="Verbatim(SUBSCRIPT, (Verbatim(SYMBOL, 'a'), (Verbatim(SLICE, (Verbatim(SYMBOL, 'b'), Verbatim(SYMBOL, 'c'), Verbatim(SPECIAL, None))),)))", repr(Verbatim('a[b:c]'))

    assert repr(Verbatim('{a:b, c:d}'))=="Verbatim(DICT, ((Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')), (Verbatim(SYMBOL, 'c'), Verbatim(SYMBOL, 'd'))))", repr(Verbatim('{a:b, c:d}'))
    assert repr(Verbatim('{a:b}'))=="Verbatim(DICT, ((Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')),))", repr(Verbatim('{a:b}'))
    assert repr(Verbatim('a.b'))=="Verbatim(ATTR, (Verbatim(SYMBOL, 'a'), Verbatim(SYMBOL, 'b')))", repr(Verbatim('a.b'))
    assert repr(Verbatim('a(b=c)'))=="Verbatim(APPLY, (Verbatim(SYMBOL, 'a'), (Verbatim(KWARG, (Verbatim(SYMBOL, 'b'), Verbatim(SYMBOL, 'c'))),)))", repr(Verbatim('a(b=c)'))

    #assert repr(Verbatim('a if b else c'))=='',repr(Verbatim('a if b else c'))
    
    assert str(+a) in ["+a","a"],str(+a)
    assert str(-a) == "-a",str(-a)
    assert str(~a) == "~a",str(~a)
    assert str(a+b)== "a + b", str(a+b)
    assert str(a-b)== "a - b", str(a-b)
    assert str(a*b)== "a*b", str(a*b)
    assert str(a/b)== "a/b", str(a/b)
    assert str(a//b)== "a//b", str(a//b)
    assert str(a**b)== "a**b", str(a**b)
    assert str(a%b)== "a%b", str(a%b)
    assert str(a|b)== "a|b", str(a|b)
    assert str(a&b)== "a&b", str(a&b)
    assert str(a^b)== "a^b", str(a^b)
    assert str(a<<b)== "a<<b", str(a<<b)
    assert str(a>>b)== "a>>b", str(a>>b)
    assert str(divmod(a,b))== "divmod(a, b)", str(divmod(a,b))
    assert str(a(b))== "a(b)", str(a(b))
    assert str(a())== "a()", str(a())
    assert str(a[b])== "a[b]", str(a[b])
    assert str(a[b:])== "a[b:]", str(a[b:])
    assert str(a[b:c])== "a[b:c]", str(a[b:c])
    assert str(a[b:c:d])== "a[b:c:d]", str(a[b:c:d])
    assert str(a[:c:d])== "a[:c:d]", str(a[:c:d])
    assert str(a[::d])== "a[::d]", str(a[::d])
    assert str(a[b::d])== "a[b::d]", str(a[b::d])
    assert str((a+b)[c])== "(a + b)[c]", str((a+c)[c])
    assert str((a+b)(c))== "(a + b)(c)", str((a+c)(c))
    assert str(Verbatim('a and b'))=="a and b", str(Verbatim('a and b'))
    assert str(Verbatim('a or b'))=="a or b", str(Verbatim('a or b'))
    assert str(Verbatim('a in b'))=="a in b", str(Verbatim('a in b'))
    assert str(Verbatim('a not in b'))=="a not in b", str(Verbatim('a not in b'))
    assert str(Verbatim('a is b'))=="a is b", str(Verbatim('a is b'))
    assert str(Verbatim('a is not b'))=="a is not b", str(Verbatim('a is not b'))
    assert str(Verbatim('a and b and c'))=="a and b and c", str(Verbatim('a and b and c'))
    assert str(Verbatim('()'))=="()", str(Verbatim('()'))
    assert str(Verbatim('(a,)'))=="(a,)", str(Verbatim('(a,)'))
    assert str(Verbatim('(a,b)'))=="(a, b)", str(Verbatim('(a,b)'))
    assert str(Verbatim('[]'))=="[]", str(Verbatim('[]'))
    assert str(Verbatim('[a]'))=="[a]", str(Verbatim('[a]'))
    assert str(Verbatim('[a,b]'))=="[a, b]", str(Verbatim('[a, b]'))
    assert str(Verbatim('lambda :a'))=="lambda : a", str(Verbatim('lambda : a'))
    assert str(Verbatim('lambda x:a'))=="lambda x: a", str(Verbatim('lambda x: a'))
    assert str(Verbatim('lambda x,y:a'))=="lambda x, y: a", str(Verbatim('lambda x,y: a'))
    assert str(Verbatim('lambda x=2:a'))=="lambda x=2: a", str(Verbatim('lambda x=2: a'))
    assert str(Verbatim('lambda a,x=2:a'))=="lambda a, x=2: a", str(Verbatim('lambda a, x=2: a'))
    assert str(Verbatim('lambda a,x=2,y=1:a'))=="lambda a, x=2, y=1: a", str(Verbatim('lambda a, x=2, y=1: a'))
    assert str(Verbatim('a< b'))=="a<b", str(Verbatim('a < b'))
    assert str(Verbatim('a==b'))=="a==b", str(Verbatim('a ==b'))
    assert str(Verbatim('a!= b'))=="a!=b", str(Verbatim('a!=b'))
    assert str(Verbatim('a<= b'))=="a<=b", str(Verbatim('a <= b'))
    assert str(Verbatim('a> b'))=="a>b", str(Verbatim('a > b'))
    assert str(Verbatim('a>= b'))=="a>=b", str(Verbatim('a >= b'))
    assert str(Verbatim('a<b<c'))=="a<b and b<c", str(Verbatim('a<b<c'))
    assert str(Verbatim('a<b==c'))=="a<b and b==c", str(Verbatim('a<b==c'))
    assert str(Verbatim('a[b:c]'))=="a[b:c]", str(Verbatim('a[b:c]'))
    assert str(Verbatim('a[b:c,d]'))=="a[b:c, d]", str(Verbatim('a[b:c, d]'))
    assert str(Verbatim('a[b:]'))=="a[b:]", str(Verbatim('a[b:]'))
    assert str(Verbatim('a[:b]'))=="a[:b]", str(Verbatim('a[:b]'))
    assert str(Verbatim('a[::b]'))=="a[::b]", str(Verbatim('a[::b]'))
    assert str(Verbatim('a[b::c]'))=="a[b::c]", str(Verbatim('a[b::c]'))
    assert str(Verbatim('a[b:c:d]'))=="a[b:c:d]", str(Verbatim('a[b:c:d]'))
    assert str(Verbatim('a[b::]'))=="a[b:]", str(Verbatim('a[b::]'))
    assert str(Verbatim('a[b::]'))=="a[b:]", str(Verbatim('a[b::]'))
    assert str(Verbatim('a[b,...,c]'))=="a[b, ..., c]", str(Verbatim('a[b,...,c]'))
    assert str(Verbatim('a[...]'))=="a[...]", str(Verbatim('a[...]'))
    assert str(Verbatim('{a:b}'))=="{a:b}", str(Verbatim('{a:b}'))
    assert str(Verbatim('{a:b, c:1}'))=="{a:b, c:1}", str(Verbatim('{a:b, c:1}'))
    assert str(Verbatim('a.b'))=="a.b", str(Verbatim('a.b'))
    assert str(Verbatim('(a+1).b'))=="(a + 1).b", str(Verbatim('(a+1).b'))
    assert str(a.b)=="a.b", str(a.b)
    assert str((a+1).b)=="(a + 1).b", str((a+1).b)
    assert str(a(b=c))=="a(b=c)", str(a(b=c))
    
    assert Verbatim('+a') == +a
    assert Verbatim('-a') == -a
    assert Verbatim('~a') == ~a, `Verbatim('~a')`
    assert Verbatim('a+b') == a+b
    assert Verbatim('a-b') == a-b
    assert Verbatim('a*b') == a*b
    assert Verbatim('a/b') == a/b
    assert Verbatim('a//b') == a//b
    assert Verbatim('a**b') == a**b
    assert Verbatim('a % b') == a%b
    assert Verbatim('a | b') == a|b
    assert Verbatim('a & b') == a&b
    assert Verbatim('a << b') == a<<b
    assert Verbatim('a >> b') == a>>b
    assert Verbatim('a (b)') == a(b)
    assert Verbatim('a (b, a)') == a(b,a)
    assert Verbatim('a ()') == a()
    assert Verbatim('a[b]') == a[b]
    assert Verbatim('a[b,c]') == a[b,c], `Verbatim('a[b,c]'),a[b,c]`
    assert Verbatim('a[b:c,d]') == a[b:c,d], `Verbatim('a[b:c,d]'),a[b:c,d]`

    assert Verbatim('a and b') == Verbatim(heads.AND, (a, b))
    assert Verbatim('a or b') == Verbatim(heads.OR, (a, b))
    assert Verbatim('not a') == Verbatim(heads.NOT, a)
    assert Verbatim('a in b') == Verbatim(heads.IN, (a, b))
    assert Verbatim('a not in b') == Verbatim(heads.NOTIN, (a, b))
    assert Verbatim('a is b') == Verbatim(heads.IS, (a, b))
    assert Verbatim('a is not b') == Verbatim(heads.ISNOT, (a, b))
    assert Verbatim('a and b and c') == Verbatim(heads.AND, (a, b, c))
    assert Verbatim('()') == Verbatim(heads.TUPLE, ())
    assert Verbatim('(a,)') == Verbatim(heads.TUPLE, (a, ))
    assert Verbatim('(a,b)') == Verbatim(heads.TUPLE, (a, b))
    assert Verbatim('[]') == Verbatim(heads.LIST, ())
    assert Verbatim('[a]') == Verbatim(heads.LIST, (a,))
    assert Verbatim('[a,b]') == Verbatim(heads.LIST, (a,b))
    assert Verbatim('lambda :a') == Verbatim(heads.LAMBDA, ((), a))
    assert Verbatim('lambda b:a') == Verbatim(heads.LAMBDA, ((b,), a))
    assert Verbatim('a==b')==Verbatim(heads.EQ, (a, b)), Verbatim('a == b')
    assert Verbatim('a!=b')==Verbatim(heads.NE, (a, b)), Verbatim('a != b')
    assert Verbatim('a<b')==Verbatim(heads.LT, (a, b)), Verbatim('a < b')
    assert Verbatim('a<=b')==Verbatim(heads.LE, (a, b)), Verbatim('a <= b')
    assert Verbatim('a>b')==Verbatim(heads.GT, (a, b)), Verbatim('a > b')
    assert Verbatim('a>=b')==Verbatim(heads.GE, (a, b)), Verbatim('a >= b')
    assert Verbatim('a[b]')==Verbatim(heads.SUBSCRIPT, (a, (b,))), Verbatim('a[b]')
    assert Verbatim('a[b,c]')==Verbatim(heads.SUBSCRIPT, (a, (b, c))), Verbatim('a[b,c]')
    assert Verbatim('a[b:c]')==Verbatim(heads.SUBSCRIPT, (a, (Verbatim(heads.SLICE, (b,c,Verbatim(heads.SPECIAL, None))),))), `Verbatim('a[b:c]')`
    assert Verbatim('a[b:c,d]')==Verbatim(heads.SUBSCRIPT, (a, (Verbatim(heads.SLICE, (b,c,Verbatim(heads.SPECIAL, None))), d))), `Verbatim('a[b:c,d]')`
    assert Verbatim('a[...]')==Verbatim(heads.SUBSCRIPT, (Verbatim(heads.SYMBOL, 'a'), (Verbatim(heads.SPECIAL, Ellipsis),))), `Verbatim('a[...]')`
    assert Verbatim('{a:b}')==Verbatim(heads.DICT, ((a, b),))
    assert Verbatim('a.b')==a.b
    assert Verbatim('a(b=c)')==a(b=c)
    assert Verbatim('a(b,c=d)')==a(b,c=d)

def test_trivial_operations():
    a = Verbatim('a')
    b = Verbatim('b')
    assert str(Verbatim(heads.ADD, []))=='0'
    assert str(Verbatim(heads.ADD, [a]))=='a'
    assert str(Verbatim(heads.SUB, []))=='0'
    assert str(Verbatim(heads.SUB, [a]))=='a'
    
def test_mixed_operations():
    a = Verbatim('a')
    b = Verbatim('b')
    c = Verbatim('c')
    d = Verbatim('d')
    m2 = Verbatim(heads.NUMBER, -2)
    
    assert str(a-(b+c))=='a - (b + c)',str(a-(b+c))
    assert str(a-b-c)=='a - b - c',str(a-b-c)
    assert str(a-(b-c))=='a - (b - c)',str(a-(b-c))
    assert str(a-(-b)) in ['a - -b','a + b'],str(a-(-b))
    assert str(a+(-b))=='a - b',str(a+(-b))
    assert str(a+(+b))=='a + b',str(a+(+b))
    assert str(a-(+b))=='a - b',str(a-(+b))
    assert str(a-(+b+c)) in ['a - (+b + c)','a - (b + c)'],str(a-(+b+c))
    assert str(a+(m2*b))=='a - 2*b',str(a+(m2*b))
    assert str(a-(-b+c))=='a - (-b + c)',str(a-(-b+c))
    assert str(a+(-b+c))=='a - b + c',str(a+(-b+c))
    assert str(a-(m2*b))=='a + 2*b',str(a-(m2*b))
    assert str(a-(b*c))=='a - b*c',str(a-(b*c))
    assert str(a+(b*c))=='a + b*c',str(a+(b*c))

    assert str(+(a+b)) in ['+a + b','a + b', '+(a + b)'],str(+(a+b))
    assert str(+(a-b)) in ['+a - b','a - b', '+(a - b)'],str(+(a-b))
    assert str(-(a+b)) in '-(a + b)',str(-(a+b))
    assert str(-(-a)) in ['a','--a'],str(-(-a))
    assert str(-(+a))=='-a',str(-(+a))

def test_subs():
    a = Verbatim('a')
    b = Verbatim('b')
    c = Verbatim('c')
    
    assert str(a[b].subs(b,c))=='a[c]'
    assert str(a[b].subs(a,c))=='c[b]'
    assert str(a[b,c].subs(b,c))=='a[c, c]'



'''
.. module:: helpers
   :platform: Linux
   :synopsis: A module containing just a small number of useful functions, such as getting the current data and some special
              linear algebra routines compiled in externally.

.. moduleauthor:: Eric M. Scheffel <eric.scheffel@nottingham.edu.cn>


'''

import datetime
import time
import copy as COP
import os as OPS
import sys as SYST
import re as RE
import string as STR
import scipy as S
from numpy import *
from numpy import matlib as MAT
from scipy import linalg as LIN
from numpy.ctypeslib import load_library, ndpointer
from ctypes import cdll, c_int, c_char, POINTER


def null(A, eps = 1e-15):
	"""
	Function to get the nullspace of a matrix.

	Usage:
	B = nullspace(A,eps)

	where A is some matrix, B is the returned
	corresponding nullspace and eps is the optional
	machine precision (smallest number).
	"""
	u, s, vh = LIN.svd(A)
	null_mask = (s <= eps)
	null_space = compress(null_mask, vh, axis=0)
	return null_space

def eig(A,B):
	"""
	To ensure matlab compatibility, we need to
	swap matrices A and B around !!
	"""
	(XX1,XX2) = LIN.eig(A,B)
	(XX3,XX4) = LIN.eig(B,A)
	return (mat(XX4),mat(XX1))

def sortrows(A, integer):
	"""
	Sort the rows of a matrix using the
	column given by the index number
	"""
	A_array = A.A
	A_array = A_array[A_array[:,integer].argsort(),]
	return mat(A_array)

def ctr(A):
	"""
	returns conjugate transpose
	"""
	return A.conjugate().transpose()

def eig2(A,B, lapackname='', lapackpath=''):
	'''Calculates generalized eigenvalues of pair (A,B).

    This should correspond to Matlab's lambda = eig(A,B),
    and also to some (the same?) scipy function.

    Eigenvalues will be of complex type, are unsorted, and are returned as 1d.
    '''
	AA,BB,dum1,dum2,VSL,VSR = zgges4numpy(A,B,
													  lapackname=lapackname,lapackpath=lapackpath)
	return diag(AA)/diag(BB)

# Time Stamp Stuff
def now_is():
	t = datetime.datetime.now()
	EpochSeconds = time.mktime(t.timetuple())
	now = datetime.datetime.fromtimestamp(EpochSeconds)
	return now.__str__()



import numpy
import copy
from ..filters import hpfilter, bkfilter, cffilter


def stdX(self,data=None,standard=False,only_demean=False,func=False):
    '''
    Function used to standardize the original data, i.e. de-mean and divide by SD,
    also saves data's characteristics upon normalizing.
    
    :param data:             The data to be normalizes/transformed
    :type data:              arr2d
    :param standard:         Should data be normalized by standard deviation?
    :type standard:          bool
    :param only_demean:      Should data only be de-meaned but not divided by std?
    :type only_demean:       bool
    :param func:             If true then method is behaving like a normal functions which returns
    :type func:              bool
    
    :return self.tdata:      *(arr2d)* - The transformed data
    :return self.nstdata:    *(arr2d)* - The nonstandardized or raw data which was passed to the method
    :return self.trans_dic:  *(dic)* - A dictionary containing the mean and std of the data. Uses vnames.
    '''
    if data != None:
        data = copy.deepcopy(data)
    else:
        data = copy.deepcopy(self.data)
    vnames = self.vnames
    trans_dic = {}
    data_nonstd = copy.deepcopy(data)
    for colo in range(0,data.shape[1],1):
        trans_dic[vnames[colo]] = {}
        mean_i = numpy.average(data[:,colo])
        trans_dic[vnames[colo]]['mean'] = copy.deepcopy(mean_i)
        if standard:
            data[:,colo] = data[:,colo] - mean_i
        std_i = numpy.std(data[:,colo])
        trans_dic[vnames[colo]]['std'] = copy.deepcopy(std_i)
        data_nonstd[:,colo] = copy.deepcopy(data[:,colo])
        if standard and not only_demean: data[:,colo] = data[:,colo]/std_i
    if not func:
        self.tdata = copy.deepcopy(data)
        self.nstddata = copy.deepcopy(data_nonstd)
        self.trans_dic = copy.deepcopy(trans_dic)
        return self
    elif func:
        return data,trans_dic
    
def transX(self,data=None,func=False):
    '''
    Function to transform the raw data according to some classification
    1 = levels
    2 = first seasonal difference
    3 = second seasonal difference
    4 = log level
    5 = log first seasonal difference
    6 = log second seasonal difference
    7 = detrend log using hp filter monthly data
    8 = detrend log using hp filter quarterly data
    16 = log second seasonal difference
    17 = (1-L)(1-L^12)
    18 = log of (1-L)*annualizing factor (i.e. x4 for quarterly and x12 for monthly
    19 = linear detrend of level (raw) data
    20 = linear detrend of log data
    21 = quadratic detrend of level (raw) data
    22 = quadratic detrend of log data
    '''
    if data != None:
        data = copy.deepcopy(data)
    elif data == None:
        data = copy.deepcopy(self.data)
    vnames = self.vnames
    freq = self.confdic['freq']
    tcode = self.confdic['tcode']
    shift = 0
    freqshift = 1
    if freq == 'M':
        freqshift = 12
    elif freq == 'Q':
        freqshift = 4
    else:
        freqshift = 1
    # for level do nothing
    # first seasonal difference
    for i1,vname in enumerate(vnames):
        if tcode[vname] == 2:
            data[1*freqshift:,i1] = data[1*freqshift:,i1]-data[:-1*freqshift,i1]
            if shift < 1: shift = 1*freqshift
        elif tcode[vname] == 3:
            data[1*freqshift:,i1] = data[1*freqshift:,i1]-data[:-1*freqshift,i1]
            data[2*freqshift:,i1] = data[2*freqshift:,i1]-data[:-2*freqshift,i1]
            if shift < 2: shift = 2*freqshift
        elif tcode[vname] == 4:
            data[:,i1] = numpy.log(data[:,i1])
        elif tcode[vname] == 5:
            data[:,i1] = numpy.log(data[:,i1])
            data[1*freqshift:,i1] = data[1*freqshift:,i1]-data[:-1*freqshift,i1]
            if shift < 1: shift = 1*freqshift
        elif tcode[vname] == 6:
            data[:,i1] = numpy.log(data[:,i1])
            data[1*freqshift:,i1] = data[1*freqshift:,i1]-data[:-1*freqshift,i1]
            data[2*freqshift:,i1] = data[2*freqshift:,i1]-data[:-2*freqshift,i1]
            if shift < 2: shift = 2*freqshift
        elif tcode[vname] == 7:
            data[:,i1] = numpy.log(data[:,i1])
            data[:,i1] = hpfilter(data=data[:,i1],lam=129600)[0]
        elif tcode[vname] == 8:
            data[:,i1] = numpy.log(data[:,i1])
            data[:,i1] = hpfilter(data=data[:,i1],lam=1600)[0]
        elif tcode[vname] == 18:
            data[:,i1] = numpy.log(data[:,i1])
            if freq == 'Q':
                data[1:,i1] = (data[1:,i1]-data[:-1,i1])*4.0
            elif freq == 'M':
                data[1:,i1] = (data[1:,i1]-data[:-1,i1])*12.0
            if shift < 1: shift = 1
        elif tcode[vname] == 19:
            data[:,i1] = scipy.signal.filter(data[:,i1], type='linear')
        elif tcode[vname] == 20:
            data[:,i1] = scipy.signal.filter(numpy.log(data[:,i1]), type='linear')
        elif tcode[vname] == 21:
            data[:,i1] = scipy.signal.bsplines.quadratic(data[:,i1])
        elif tcode[vname] == 22:
            data[:,i1] = scipy.signal.bsplines.quadratic(numpy.log(data[:,i1]))
    if shift > 0: data = data[shift:,:]
    if not func:
        self.max_shift = shift
        self.data = copy.deepcopy(data)
        return self
    else:
        return data



__all__ = ['INTEGER_LIST']

from .base import Head

class IntegerListHead(Head):

    def __repr__(self): return 'INTEGER_LIST'

    def is_data_ok(self, cls, data):
        if type(data) is not list:
            return 'data must be list but got %r' % (type(data).__name__) #pragma: no cover
        for i, item in enumerate(data):
            if not isinstance(item, (int, long)): #pragma: no cover
                return 'all data items must be integers but %s-th item is %r' % (i, type(item).__name__) #pragma: no cover

    def to_lowlevel(self, cls, data, pair):
        if len(data)==1:
            return data[0]
        return pair

INTEGER_LIST = IntegerListHead()



from mpmath import *

def test_diff():
    mp.dps = 15
    assert diff(log, 2.0, n=0).ae(log(2))
    assert diff(cos, 1.0).ae(-sin(1))
    assert diff(abs, 0.0) == 0
    assert diff(abs, 0.0, direction=1) == 1
    assert diff(abs, 0.0, direction=-1) == -1
    assert diff(exp, 1.0).ae(e)
    assert diff(exp, 1.0, n=5).ae(e)
    assert diff(exp, 2.0, n=5, direction=3*j).ae(e**2)
    assert diff(lambda x: x**2, 3.0, method='quad').ae(6)
    assert diff(lambda x: 3+x**5, 3.0, n=2, method='quad').ae(540)
    assert diff(lambda x: 3+x**5, 3.0, n=2, method='step').ae(540)
    assert diffun(sin)(2).ae(cos(2))
    assert diffun(sin, n=2)(2).ae(-sin(2))

def test_taylor():
    mp.dps = 15
    # Easy to test since the coefficients are exact in floating-point
    assert taylor(sqrt, 1, 4) == [1, 0.5, -0.125, 0.0625, -0.0390625]

def test_diff_partial():
    mp.dps = 15
    x,y,z = xyz = 2,3,7
    f = lambda x,y,z: 3*x**2 * (y+2)**3 * z**5
    assert diff(f, xyz, (0,0,0)).ae(25210500)
    assert diff(f, xyz, (0,0,1)).ae(18007500)
    assert diff(f, xyz, (0,0,2)).ae(10290000)
    assert diff(f, xyz, (0,1,0)).ae(15126300)
    assert diff(f, xyz, (0,1,1)).ae(10804500)
    assert diff(f, xyz, (0,1,2)).ae(6174000)
    assert diff(f, xyz, (0,2,0)).ae(6050520)
    assert diff(f, xyz, (0,2,1)).ae(4321800)
    assert diff(f, xyz, (0,2,2)).ae(2469600)
    assert diff(f, xyz, (1,0,0)).ae(25210500)
    assert diff(f, xyz, (1,0,1)).ae(18007500)
    assert diff(f, xyz, (1,0,2)).ae(10290000)
    assert diff(f, xyz, (1,1,0)).ae(15126300)
    assert diff(f, xyz, (1,1,1)).ae(10804500)
    assert diff(f, xyz, (1,1,2)).ae(6174000)
    assert diff(f, xyz, (1,2,0)).ae(6050520)
    assert diff(f, xyz, (1,2,1)).ae(4321800)
    assert diff(f, xyz, (1,2,2)).ae(2469600)
    assert diff(f, xyz, (2,0,0)).ae(12605250)
    assert diff(f, xyz, (2,0,1)).ae(9003750)
    assert diff(f, xyz, (2,0,2)).ae(5145000)
    assert diff(f, xyz, (2,1,0)).ae(7563150)
    assert diff(f, xyz, (2,1,1)).ae(5402250)
    assert diff(f, xyz, (2,1,2)).ae(3087000)
    assert diff(f, xyz, (2,2,0)).ae(3025260)
    assert diff(f, xyz, (2,2,1)).ae(2160900)
    assert diff(f, xyz, (2,2,2)).ae(1234800)
